package com.isu.gaswellwatch.vo.command.weps;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 威尔普斯控制指令
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2025/1/10 11:00
 */
@Getter
@Setter
@SuperBuilder
public class WepsPlugControl extends Command implements Timing {
    @Serial
    private static final long serialVersionUID = -7540015879950965127L;

    private Integer runMode;
    private Integer wellStatus;
    private Integer gasCollectionMethod;
    private BigDecimal highPressureProtection;
    private BigDecimal lowPressureProtection;
    private Integer tooFastCount;
    private String dangerousRiseTime;
    private String tooFastRiseTime;
    private BigDecimal openWellTime;
    private BigDecimal closeWellTime;
    private BigDecimal openWellCasPressure;
    private BigDecimal littleRiseCasPressure;
    private String checkStabilityTime;
    private String minCloseWellTime;
    private String maxCloseWellTime;
    private String minOpenWellTime;
    private String maxOpenWellTime;
    private Integer wellDepth;
    private Integer produceMode;

    public WepsPlugControl() {
        this.setCode("WEPS.PLUG.CONTROL");
    }

    public void setDangerousRiseTime(String dangerousRiseTime) {
        this.dangerousRiseTime = this.timingValidate(dangerousRiseTime, "dangerousRiseTime");
    }

    public void setTooFastRiseTime(String tooFastRiseTime) {
        this.tooFastRiseTime = this.timingValidate(tooFastRiseTime, "tooFastRiseTime");
    }

    public void setCheckStabilityTime(String checkStabilityTime) {
        this.checkStabilityTime = this.timingValidate(checkStabilityTime, "checkStabilityTime");
    }

    public void setMinCloseWellTime(String minCloseWellTime) {
        this.minCloseWellTime = this.timingValidate(minCloseWellTime, "minCloseWellTime");
    }

    public void setMaxCloseWellTime(String maxCloseWellTime) {
        this.maxCloseWellTime = this.timingValidate(maxCloseWellTime, "maxCloseWellTime");
    }

    public void setMinOpenWellTime(String minOpenWellTime) {
        this.minOpenWellTime = this.timingValidate(minOpenWellTime, "minOpenWellTime");
    }

    public void setMaxOpenWellTime(String maxOpenWellTime) {
        this.maxOpenWellTime = this.timingValidate(maxOpenWellTime, "maxOpenWellTime");
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {

        StringBuilder command = new StringBuilder(250);
        //               100
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     003F     0013    26
        command.append("0110003F001326");
        command.append(StringUtils.leftPad(Integer.toHexString(this.runMode), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellStatus), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.gasCollectionMethod), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.highPressureProtection.multiply(ONE_HUNDRED).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.lowPressureProtection.multiply(ONE_HUNDRED).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.tooFastCount), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.dangerousRiseTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.tooFastRiseTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.openWellTime.multiply(ONE_SIXTY).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.closeWellTime.multiply(ONE_SIXTY).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.openWellCasPressure.multiply(ONE_HUNDRED).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.littleRiseCasPressure.multiply(ONE_HUNDRED).longValue()), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.checkStabilityTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.minCloseWellTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.maxCloseWellTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.minOpenWellTime)), 4, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toMinutes(this.maxOpenWellTime)), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellDepth), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.produceMode), 4, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

}
